# SPDX-License-Identifier: CECILL-C

const MLS_LANG_CODE = Dict(
    "deu" => "german",
    "eng" => "english",
    "esp" => "spanish",
    "fra" => "french",
    "ita" => "italian",
    "nld" => "dutch",
    "pol" => "polish",
    "prt" => "portuguese"
)

const MLS_AUDIO_URLS = Dict(
    "deu" => "https://dl.fbaipublicfiles.com/mls/mls_german.tar.gz",
    "eng" => "https://dl.fbaipublicfiles.com/mls/mls_english.tar.gz",
    "esp" => "https://dl.fbaipublicfiles.com/mls/mls_spanish.tar.gz",
    "fra" => "https://dl.fbaipublicfiles.com/mls/mls_french.tar.gz",
    "ita" => "https://dl.fbaipublicfiles.com/mls/mls_italian.tar.gz",
    "nld" => "https://dl.fbaipublicfiles.com/mls/mls_dutch.tar.gz",
    "pol" => "https://dl.fbaipublicfiles.com/mls/mls_polish.tar.gz",
    "prt" => "https://dl.fbaipublicfiles.com/mls/mls_portuguese.tar.gz"
)

const MLS_LM_URLS = Dict(
    "deu" => "https://dl.fbaipublicfiles.com/mls/mls_lm_german.tar.gz",
    "eng" => "https://dl.fbaipublicfiles.com/mls/mls_lm_english.tar.gz",
    "esp" => "https://dl.fbaipublicfiles.com/mls/mls_lm_spanish.tar.gz",
    "fra" => "https://dl.fbaipublicfiles.com/mls/mls_lm_french.tar.gz",
    "ita" => "https://dl.fbaipublicfiles.com/mls/mls_lm_italian.tar.gz",
    "nld" => "https://dl.fbaipublicfiles.com/mls/mls_lm_dutch.tar.gz",
    "pol" => "https://dl.fbaipublicfiles.com/mls/mls_lm_polish.tar.gz",
    "prt" => "https://dl.fbaipublicfiles.com/mls/mls_lm_portuguese.tar.gz"
)

const supported_lang = collect(keys(MLS_LANG_CODE))

struct MultilingualLibriSpeech <: SpeechCorpus
    lang::AbstractString
    name::AbstractString
end

function MultilingualLibriSpeech(lang)
    if ! (lang in supported_lang)
        throw(ArgumentError("lang must be one of $supported_lang"))
    end 
    MultilingualLibriSpeech(lang, "multilingual_librispeech")
end

function Base.download(corpus::MultilingualLibriSpeech, dir)
    donefile = joinpath(dir, ".download.done")
    if ! isfile(donefile)
        run(`mkdir -p $dir`)
        @info "downloading the corpus"
        run(`wget -P $dir $(MLS_AUDIO_URLS[corpus.lang])`)
        tarpath = joinpath(dir, "mls_$(MLS_LANG_CODE[corpus.lang]).tar.gz")
        @info "extracting"
        run(`tar -xf $tarpath -C $dir`)
        run(`rm $tarpath`)

        @info "downloading LM data"
        run(`wget -P $dir $(MLS_LM_URLS[corpus.lang])`)
        tarpath = joinpath(dir, "mls_lm_$(MLS_LANG_CODE[corpus.lang]).tar.gz")
        @info "extracting"
        run(`tar -xf $tarpath -C $dir`)
        run(`rm $tarpath`)

        run(pipeline(`date`, stdout = donefile))
    end
    @info "dataset in $dir"

    corpus
end

function mlls_recordings(corpus::MultilingualLibriSpeech, inputdir, subset)
    subsetdir = joinpath(inputdir, "mls_$(MLS_LANG_CODE[corpus.lang])", subset, "audio")
    recs = Dict()

    for d1 in readdir(subsetdir; join = true)
        for d2 in readdir(d1; join = true)
            for path in readdir(d2; join = true)
                id = replace(basename(path), ".flac" =>  "")
                r = Recording(
                    id,
                    AudioSources.CmdAudioSource(`sox $path -t wav -`);
                    channels = [1],
                    samplerate = 16000
                )
                recs[r.id] = r
            end
        end
    end
    recs
end

function mlls_annotations(corpus::MultilingualLibriSpeech, inputdir, subset)
    trans = joinpath(inputdir, "mls_$(MLS_LANG_CODE[corpus.lang])", subset, "transcripts.txt")
    sups = Dict()
    open(trans, "r") do f
        for line in eachline(f)
            tokens = split(line)
            s = Annotation(tokens[1], tokens[1]; channels = [1],
                            data = Dict("text" => join(tokens[2:end], " ")))
            sups[s.id] = s
        end
    end
    sups
end

function mlls_prepare(corpus::MultilingualLibriSpeech, inputdir, outputdir)
    outputdir = mkpath(outputdir)

    # 1. Recording manifests.
    
    out = joinpath(outputdir, "recordings.jsonl")
    @info "preparing recording manifest $out"
    if ! isfile(out)
        open(out, "a") do f
            for subset in ["train", "dev", "test"]
                recs = mlls_recordings(corpus, inputdir, subset)
                writemanifest(f, recs)
            end
        end
    end

    # 2. Annotation manifests.
    for subset in ["train", "dev", "test"]
        out = joinpath(outputdir, "annotations-$subset.jsonl")
        @info "preparing annotation manifest ($subset) $out"
        if ! isfile(out)
            sups = mlls_annotations(corpus, inputdir, subset)
            open(out, "w") do f
                writemanifest(f, sups)
            end
        end
    end

    corpus
end

"""
    MLLIBRISPEECH(datadir::AbstractString, outputdir::AbstractString, subset::AbstractString, lang::AbstractString)
Extract metadata and paths from Multilingual LibriSpeech dataset.\n
# Arguments
- `datadir` is the name of dataset directory. If the directory does not exists, it is created and the data is downloaded.\n
- `outputdir` is the output directory for manifest files.
- `subset` must be one of ["train", "dev", "test"].\n
- `lang` is the language, must be one of $supported_lang.\n
Create the `outputdir` folder, with:
- `recordings-<subset>.jsonl` containing each audio file path and associated metadata
- `annotations-<subset>.jsonl` containing each annotation and associated metadata
Return a SpeechDataset object.
"""
function MLLIBRISPEECH(datadir::AbstractString, outputdir::AbstractString, subset::AbstractString, lang::AbstractString)
    mlls = MultilingualLibriSpeech(lang)
    # download if no datadir 
    if ! isdir(datadir)
        download(mlls, datadir)
    end
    # prepare if not already
    if ! (isfile(joinpath(outputdir, "recordings.jsonl")) &&
        isfile(joinpath(outputdir, "annotations-train.jsonl")) &&
        isfile(joinpath(outputdir, "annotations-dev.jsonl")) &&
        isfile(joinpath(outputdir, "annotations-test.jsonl")))
        mlls_prepare(mlls, datadir, outputdir)
  end
    infos = SpeechDatasetInfos("Multilingual LibriSpeech")
    SpeechDataset(infos, outputdir, subset)
end